ENUM_VALUE(GetDeviceInfo, 0x1001)
ENUM_VALUE(OpenSession, 0x1002)
ENUM_VALUE(CloseSession, 0x1003)
ENUM_VALUE(GetStorageIDs, 0x1004)
ENUM_VALUE(GetStorageInfo, 0x1005)
ENUM_VALUE(GetNumObjects, 0x1006)
ENUM_VALUE(GetObjectHandles, 0x1007)
ENUM_VALUE(GetObjectInfo, 0x1008)
ENUM_VALUE(GetObject, 0x1009)
ENUM_VALUE(GetThumb, 0x100a)
ENUM_VALUE(DeleteObject, 0x100b)
ENUM_VALUE(SendObjectInfo, 0x100c)
ENUM_VALUE(SendObject, 0x100d)
ENUM_VALUE(InitiateCapture, 0x100e)
ENUM_VALUE(FormatStore, 0x100f)
ENUM_VALUE(ResetDevice, 0x1010)
ENUM_VALUE(SelfTest, 0x1011)
ENUM_VALUE(SetObjectProtection, 0x1012)
ENUM_VALUE(PowerDown, 0x1013)
ENUM_VALUE(GetDevicePropDesc, 0x1014)
ENUM_VALUE(GetDevicePropValue, 0x1015)
ENUM_VALUE(SetDevicePropValue, 0x1016)
ENUM_VALUE(ResetDevicePropValue, 0x1017)
ENUM_VALUE(TerminateOpenCapture, 0x1018)
ENUM_VALUE(MoveObject, 0x1019)
ENUM_VALUE(CopyObject, 0x101a)
ENUM_VALUE(GetPartialObject, 0x101b)
ENUM_VALUE(InitiateOpenCapture, 0x101c)

ENUM_VALUE(CancelTransaction, 0x4001)

ENUM_VALUE(JanusGetSecureTimeChallenge, 0x9101)
ENUM_VALUE(JanusSetSecureTimeResponse, 0x9102)
ENUM_VALUE(JanusSetLicenseResponse, 0x9103)
ENUM_VALUE(JanusGetSyncList, 0x9104)
ENUM_VALUE(JanusSendMeterChallengeQuery, 0x9105)
ENUM_VALUE(JanusGetMeterChallenge, 0x9106)
ENUM_VALUE(JanusSetMeterResponse, 0x9107)
ENUM_VALUE(JanusCleanDataStore, 0x9108)
ENUM_VALUE(JanusGetLicenseState, 0x9109)
ENUM_VALUE(JanusSendCommand, 0x910a)
ENUM_VALUE(JanusSendRequest, 0x910b)
ENUM_VALUE(ProcessWCFObject, 0x9122)

ENUM_VALUE(OpenMediaSession, 0x9170)
ENUM_VALUE(CloseMediaSession, 0x9171)
ENUM_VALUE(GetNextDataBlock, 0x9172)
ENUM_VALUE(SetCurrentTimePosition, 0x9173)

ENUM_VALUE(WMPMetadataRoundTrip, 0x9201)
ENUM_VALUE(WmpGetAcquiredContent, 0x9202)

ENUM_VALUE(SendWMDRMPDAppRequest, 0x9212)
ENUM_VALUE(GetWMDRMPDAppResponse, 0x9213)
ENUM_VALUE(EnableTrustedFilesOperations, 0x9214)
ENUM_VALUE(DisableTrustedFilesOperations, 0x9215)
ENUM_VALUE(EndTrustedAppSession, 0x9216)

ENUM_VALUE(SendTinyCLRData, 0x9401)
ENUM_VALUE(GetTinyCLRData, 0x9402)

ENUM_VALUE(GetPartialObject64, 0x95c1)
ENUM_VALUE(SendPartialObject, 0x95c2)
ENUM_VALUE(TruncateObject, 0x95c3)
ENUM_VALUE(BeginEditObject, 0x95c4)
ENUM_VALUE(EndEditObject, 0x95c5)

ENUM_VALUE(GetObjectPropsSupported, 0x9801)
ENUM_VALUE(GetObjectPropDesc, 0x9802)
ENUM_VALUE(GetObjectPropValue, 0x9803)
ENUM_VALUE(SetObjectPropValue, 0x9804)
ENUM_VALUE(GetObjectPropList, 0x9805)
ENUM_VALUE(SetObjectPropList, 0x9806)
ENUM_VALUE(GetInterdependentPropDesc, 0x9807)
ENUM_VALUE(SendObjectPropList, 0x9808)
ENUM_VALUE(GetObjectReferences, 0x9810)
ENUM_VALUE(SetObjectReferences, 0x9811)
ENUM_VALUE(UpdateDeviceFirmware, 0x9812)
ENUM_VALUE(ResetObjectPropValue, 0x9813)

ENUM_VALUE(Skip, 0x9820)
