ENUM_VALUE(OK, 0x2001)
ENUM_VALUE(GeneralError, 0x2002)
ENUM_VALUE(SessionNotOpen, 0x2003)
ENUM_VALUE(InvalidTransaction, 0x2004)
ENUM_VALUE(OperationNotSupported, 0x2005)
ENUM_VALUE(ParameterNotSupported, 0x2006)
ENUM_VALUE(IncompleteTransfer, 0x2007)
ENUM_VALUE(InvalidStorageID, 0x2008)
ENUM_VALUE(InvalidObjectHandle, 0x2009)
ENUM_VALUE(DevicePropNotSupported, 0x200a)
ENUM_VALUE(InvalidObjectFormatCode, 0x200b)
ENUM_VALUE(StoreFull, 0x200c)
ENUM_VALUE(ObjectWriteProtected, 0x200d)
ENUM_VALUE(StoreReadOnly, 0x200e)
ENUM_VALUE(AccessDenied, 0x200f)
ENUM_VALUE(NoThumbnailPresent, 0x2010)
ENUM_VALUE(SelfTestFailed, 0x2011)
ENUM_VALUE(PartialDeletion, 0x2012)
ENUM_VALUE(StoreNotAvailable, 0x2013)
ENUM_VALUE(SpecificationByFormatUnsupported, 0x2014)
ENUM_VALUE(NoValidObjectInfo, 0x2015)
ENUM_VALUE(InvalidCodeFormat, 0x2016)
ENUM_VALUE(UnknownVendorCode, 0x2017)
ENUM_VALUE(CaptureAlreadyTerminated, 0x2018)
ENUM_VALUE(DeviceBusy, 0x2019)
ENUM_VALUE(InvalidParentObject, 0x201a)
ENUM_VALUE(InvalidDevicePropFormat, 0x201b)
ENUM_VALUE(InvalidDevicePropValue, 0x201c)
ENUM_VALUE(InvalidParameter, 0x201d)
ENUM_VALUE(SessionAlreadyOpen, 0x201e)
ENUM_VALUE(TransactionCancelled, 0x201f)
ENUM_VALUE(SpecificationOfDestinationUnsupported, 0x2020)
ENUM_VALUE(TinyCLRNotResponding, 0xa401)
ENUM_VALUE(NoDataWaiting, 0xa402)
ENUM_VALUE(InvalidObjectPropCode, 0xa801)
ENUM_VALUE(InvalidObjectPropFormat, 0xa802)
ENUM_VALUE(InvalidObjectPropValue, 0xa803)
ENUM_VALUE(InvalidObjectReference, 0xa804)
ENUM_VALUE(GroupNotSupported, 0xa805)
ENUM_VALUE(InvalidDataset, 0xa806)
ENUM_VALUE(UnsupportedSpecByGroup, 0xa807)
ENUM_VALUE(UnsupportedSpecByDepth, 0xa808)
ENUM_VALUE(ObjectTooLarge, 0xa809)
ENUM_VALUE(ObjectPropNotSupported, 0xa80)
