ENUM_VALUE(BuyFlag, 0xd901)
ENUM_VALUE(HostEUI64Array, 0xd920)

ENUM_VALUE(ContentTypeUUID, 0xda97) //see WPD_OBJECT_CONTENT_TYPE
ENUM_VALUE(ArtistId, 0xdab9)
ENUM_VALUE(AlbumId, 0xdabb)
ENUM_VALUE(StorageId, 0xdc01)
ENUM_VALUE(ObjectFormat, 0xdc02)
ENUM_VALUE(ProtectionStatus, 0xdc03)
ENUM_VALUE(ObjectSize, 0xdc04)
ENUM_VALUE(AssociationType, 0xdc05)
ENUM_VALUE(AssociationDesc, 0xdc06)
ENUM_VALUE(ObjectFilename, 0xdc07)
ENUM_VALUE(DateCreated, 0xdc08)
ENUM_VALUE(DateModified, 0xdc09)
ENUM_VALUE(Keywords, 0xdc0a)
ENUM_VALUE(ParentObject, 0xdc0b)
ENUM_VALUE(AllowedFolderContents, 0xdc0c)
ENUM_VALUE(Hidden, 0xdc0d)
ENUM_VALUE(SystemObject, 0xdc0e)

ENUM_VALUE(PersistentUniqueObjectId, 0xdc41)
ENUM_VALUE(SyncId, 0xdc42)
ENUM_VALUE(PropertyBag, 0xdc43)
ENUM_VALUE(Name, 0xdc44)
ENUM_VALUE(CreatedBy, 0xdc45)
ENUM_VALUE(Artist, 0xdc46)
ENUM_VALUE(DateAuthored, 0xdc47)
ENUM_VALUE(Description, 0xdc48)
ENUM_VALUE(UrlReference, 0xdc49)
ENUM_VALUE(LanguageLocale, 0xdc4a)
ENUM_VALUE(CopyrightInformation, 0xdc4b)
ENUM_VALUE(Source, 0xdc4c)
ENUM_VALUE(OriginLocation, 0xdc4d)
ENUM_VALUE(DateAdded, 0xdc4e)
ENUM_VALUE(NonConsumable, 0xdc4f)
ENUM_VALUE(CorruptedNonPlayable, 0xdc50)
ENUM_VALUE(ProducerSerialNumber, 0xdc51)

ENUM_VALUE(RepresentativeSampleFormat, 0xdc81)
ENUM_VALUE(RepresentativeSampleSize, 0xdc82)
ENUM_VALUE(RepresentativeSampleHeight, 0xdc83)
ENUM_VALUE(RepresentativeSampleWidth, 0xdc84)
ENUM_VALUE(RepresentativeSampleDuration, 0xdc85)
ENUM_VALUE(RepresentativeSampleData, 0xdc86)
ENUM_VALUE(Width, 0xdc87)
ENUM_VALUE(Height, 0xdc88)
ENUM_VALUE(Duration, 0xdc89)
ENUM_VALUE(UserRating, 0xdc8a)
ENUM_VALUE(Track, 0xdc8b)
ENUM_VALUE(Genre, 0xdc8c)
ENUM_VALUE(Credits, 0xdc8d)
ENUM_VALUE(Lyrics, 0xdc8e)
ENUM_VALUE(SubscriptionContentId, 0xdc8f)
ENUM_VALUE(ProducedBy, 0xdc90)
ENUM_VALUE(UseCount, 0xdc91)
ENUM_VALUE(SkipCount, 0xdc92)
ENUM_VALUE(LastAccessed, 0xdc93)
ENUM_VALUE(ParentalRating, 0xdc94)
ENUM_VALUE(MetaGenre, 0xdc95)
ENUM_VALUE(Composer, 0xdc96)
ENUM_VALUE(EffectiveRating, 0xdc97)
ENUM_VALUE(Subtitle, 0xdc98)
ENUM_VALUE(OriginalReleaseDate, 0xdc99)
ENUM_VALUE(AlbumName, 0xdc9a)
ENUM_VALUE(AlbumArtist, 0xdc9b)
ENUM_VALUE(Mood, 0xdc9c)
ENUM_VALUE(DrmProtectionStatus, 0xdc9d)
ENUM_VALUE(Subdescription, 0xdc9e)

ENUM_VALUE(IsCropped, 0xdcd1)
ENUM_VALUE(IsColourCorrected, 0xdcd2)
ENUM_VALUE(ImageBitDepth, 0xdcd3)
ENUM_VALUE(FNumber, 0xdcd4)
ENUM_VALUE(ExposureTime, 0xdcd5)
ENUM_VALUE(ExposureIndex, 0xdcd6)

ENUM_VALUE(TotalBitrate, 0xde91)
ENUM_VALUE(BitrateType, 0xde92)
ENUM_VALUE(SampleRate, 0xde93)
ENUM_VALUE(NumberOfChannels, 0xde94)
ENUM_VALUE(AudioBitDepth, 0xde95)
ENUM_VALUE(BlockAlignment, 0xde96)
ENUM_VALUE(ScanType, 0xde97)
ENUM_VALUE(ColourRange, 0xde98)
ENUM_VALUE(AudioWaveCodec, 0xde99)
ENUM_VALUE(AudioBitrate, 0xde9a)
ENUM_VALUE(VideoFourCCCodec, 0xde9b)
ENUM_VALUE(VideoBitrate, 0xde9c)
ENUM_VALUE(FramesPerMilliseconds, 0xde9d)
ENUM_VALUE(KeyframeDistance, 0xde9e)
ENUM_VALUE(BufferSize, 0xde9f)
ENUM_VALUE(EncodingQuality, 0xdea0)
ENUM_VALUE(EncodingProfile, 0xdea1)

ENUM_VALUE(DisplayName, 0xdce0)
ENUM_VALUE(BodyText, 0xdce1)
ENUM_VALUE(Subject, 0xdce2)
ENUM_VALUE(Priority, 0xdce3)

ENUM_VALUE(Owner, 0xdd5d)
ENUM_VALUE(Editor, 0xdd5e)
ENUM_VALUE(WebMaster, 0xdd5f)
ENUM_VALUE(UrlSource, 0xdd60)
ENUM_VALUE(UrlDestination, 0xdd61)
ENUM_VALUE(TimeBookmark, 0xdd62)
ENUM_VALUE(ObjectBookmark, 0xdd63)
ENUM_VALUE(ByteBookmark, 0xdd64)

ENUM_VALUE(LastBuildDate, 0xdd70)
ENUM_VALUE(TimeToLive, 0xdd71)
ENUM_VALUE(MediaGUID, 0xdd72)
ENUM_VALUE(All, 0xffff)
