ENUM_VALUE(Any, 0x0000)
ENUM_VALUE(Undefined, 0x3000)
ENUM_VALUE(Association, 0x3001)
ENUM_VALUE(Script, 0x3002)
ENUM_VALUE(Executable, 0x3003)
ENUM_VALUE(Text, 0x3004)
ENUM_VALUE(Html, 0x3005)
ENUM_VALUE(Dpof, 0x3006)
ENUM_VALUE(Aiff, 0x3007)
ENUM_VALUE(Wav, 0x3008)
ENUM_VALUE(Mp3, 0x3009)
ENUM_VALUE(Avi, 0x300a)
ENUM_VALUE(Mpeg, 0x300b)
ENUM_VALUE(Asf, 0x300c)

ENUM_VALUE(UndefinedImage, 0x3800)
ENUM_VALUE(ExifJpeg, 0x3801)
ENUM_VALUE(TiffEp, 0x3802)
ENUM_VALUE(Flashpix, 0x3803)
ENUM_VALUE(Bmp, 0x3804)
ENUM_VALUE(Ciff, 0x3805)
ENUM_VALUE(Gif, 0x3807)
ENUM_VALUE(Jfif, 0x3808)
ENUM_VALUE(Pcd, 0x3809)
ENUM_VALUE(Pict, 0x380a)
ENUM_VALUE(Png, 0x380b)
ENUM_VALUE(Tiff, 0x380d)
ENUM_VALUE(TiffIt, 0x380e)
ENUM_VALUE(Jp2, 0x380f)
ENUM_VALUE(Jpx, 0x3810)
ENUM_VALUE(Dng, 0x3811)
ENUM_VALUE(Heif, 0x3812)

ENUM_VALUE(NetworkAssociation, 0xb102)

ENUM_VALUE(M4a, 0xb215)
ENUM_VALUE(Artist, 0xb218)

ENUM_VALUE(UndefinedFirmware, 0xb800)
ENUM_VALUE(UndefinedFirmwareAndroid, 0xb802)
ENUM_VALUE(WindowsImageFormat, 0xb881)

ENUM_VALUE(UndefinedAudio, 0xb900)
ENUM_VALUE(Wma, 0xb901)
ENUM_VALUE(Ogg, 0xb902)
ENUM_VALUE(Aac, 0xb903)
ENUM_VALUE(Audible, 0xb904)
ENUM_VALUE(Flac, 0xb906)

ENUM_VALUE(UndefinedVideo, 0xb980)
ENUM_VALUE(Wmv, 0xb981)
ENUM_VALUE(Mp4, 0xb982)
ENUM_VALUE(Mp2, 0xb983)
ENUM_VALUE(_3gp, 0xb984)

ENUM_VALUE(UndefinedCollection, 0xba00)
ENUM_VALUE(AbstractMultimediaAlbum, 0xba01)
ENUM_VALUE(AbstractImageAlbum, 0xba02)
ENUM_VALUE(AbstractAudioAlbum, 0xba03)
ENUM_VALUE(AbstractVideoAlbum, 0xba04)
ENUM_VALUE(AbstractAVPlaylist, 0xba05)
ENUM_VALUE(AbstractContactGroup, 0xba06)
ENUM_VALUE(AbstractMessageFolder, 0xba07)
ENUM_VALUE(AbstractChapteredProduction, 0xba08)
ENUM_VALUE(AbstractAudioPlaylist, 0xba09)
ENUM_VALUE(AbstractVideoPlaylist, 0xba0a)
ENUM_VALUE(AbstractMediacast, 0xba0b)
ENUM_VALUE(WplPlaylist, 0xba10)
ENUM_VALUE(M3uPlaylist, 0xba11)
ENUM_VALUE(MplPlaylist, 0xba12)
ENUM_VALUE(AsxPlaylist, 0xba13)
ENUM_VALUE(PlsPlaylist, 0xba14)

ENUM_VALUE(UndefinedDocument, 0xba80)
ENUM_VALUE(AbstractDocument, 0xba81)
ENUM_VALUE(XmlDocument, 0xba82)
ENUM_VALUE(MicrosoftWordDocument, 0xba83)
ENUM_VALUE(MhtCompiledHtmlDocument, 0xba84)
ENUM_VALUE(MicrosoftExcelSpreadsheet, 0xba85)
ENUM_VALUE(MicrosoftPowerPointPresentation, 0xba86)

ENUM_VALUE(UndefinedMessage, 0xbb00)
ENUM_VALUE(AbstractMessage, 0xbb01)
ENUM_VALUE(UndefinedContact, 0xbb80)
ENUM_VALUE(AbstractContact, 0xbb81)
ENUM_VALUE(VCard2, 0xbb82)
ENUM_VALUE(VCard3, 0xbb83)

ENUM_VALUE(UndefinedCalendarItem, 0xbe00)
ENUM_VALUE(AbstractCalendarItem, 0xbe01)
ENUM_VALUE(VCalendar1, 0xbe02)
ENUM_VALUE(VCalendar2, 0xbe03)
ENUM_VALUE(UndefinedWindowsExecutable, 0xbe80)
