ENUM_VALUE(Undefined, 0x5000)
ENUM_VALUE(BatteryLevel, 0x5001)
ENUM_VALUE(FunctionalMode, 0x5002) //0 - normal, 1 - sleeping
ENUM_VALUE(ImageSize, 0x5003)
ENUM_VALUE(CompressionSetting, 0x5004)
ENUM_VALUE(WhiteBalance, 0x5005)
ENUM_VALUE(RgbGain, 0x5006)
ENUM_VALUE(FNumber, 0x5007)
ENUM_VALUE(FocalLength, 0x5008)
ENUM_VALUE(FocusDistance, 0x5009)
ENUM_VALUE(FocusMode, 0x500a)
ENUM_VALUE(ExposureMeteringMode, 0x500b)
ENUM_VALUE(FlashMode, 0x500c)
ENUM_VALUE(ExposureTime, 0x500d)
ENUM_VALUE(ExposureProgramMode, 0x500e)
ENUM_VALUE(ExposureIndex, 0x500f)
ENUM_VALUE(ExposureBiasCompensation, 0x5010)
ENUM_VALUE(Datetime, 0x5011)
ENUM_VALUE(CaptureDelay, 0x5012)
ENUM_VALUE(StillCaptureMode, 0x5013)
ENUM_VALUE(Contrast, 0x5014)
ENUM_VALUE(Sharpness, 0x5015)
ENUM_VALUE(DigitalZoom, 0x5016)
ENUM_VALUE(EffectMode, 0x5017)
ENUM_VALUE(BurstNumber, 0x5018)
ENUM_VALUE(BurstInterval, 0x5019)
ENUM_VALUE(TimelapseNumber, 0x501a)
ENUM_VALUE(TimelapseInterval, 0x501b)
ENUM_VALUE(FocusMeteringMode, 0x501c)
ENUM_VALUE(UploadUrl, 0x501d)
ENUM_VALUE(Artist, 0x501e)
ENUM_VALUE(CopyrightInfo, 0x501f)

ENUM_VALUE(DeviceEUI64, 0xd210)

ENUM_VALUE(SynchronizationPartner, 0xd401)
ENUM_VALUE(DeviceFriendlyName, 0xd402)
ENUM_VALUE(Volume, 0xd403)
ENUM_VALUE(SupportedFormatsOrdered, 0xd404)
ENUM_VALUE(DeviceIcon, 0xd405)
ENUM_VALUE(PlaybackRate, 0xd410)
ENUM_VALUE(PlaybackObject, 0xd411)
ENUM_VALUE(PlaybackContainerIndex, 0xd412)
ENUM_VALUE(PlaybackPosition, 0xd413)

ENUM_VALUE(SessionInitiatorVersionInfo, 0xd406)
ENUM_VALUE(PerceivedDeviceType, 0xd407)
