ENUM_VALUE(Undefined, 0x0000)
ENUM_VALUE(Int8, 0x0001)
ENUM_VALUE(Uint8, 0x0002)
ENUM_VALUE(Int16, 0x0003)
ENUM_VALUE(Uint16, 0x0004)
ENUM_VALUE(Int32, 0x0005)
ENUM_VALUE(Uint32, 0x0006)
ENUM_VALUE(Int64, 0x0007)
ENUM_VALUE(Uint64, 0x0008)
ENUM_VALUE(Int128, 0x0009)
ENUM_VALUE(Uint128, 0x000a)
ENUM_VALUE(ArrayInt8, 0x4001)
ENUM_VALUE(ArrayUint8, 0x4002)
ENUM_VALUE(ArrayInt16, 0x4003)
ENUM_VALUE(ArrayUint16, 0x4004)
ENUM_VALUE(ArrayInt32, 0x4005)
ENUM_VALUE(ArrayUint32, 0x4006)
ENUM_VALUE(ArrayInt64, 0x4007)
ENUM_VALUE(ArrayUint64, 0x4008)
ENUM_VALUE(ArrayInt128, 0x4009)
ENUM_VALUE(ArrayUint128, 0x400a)
ENUM_VALUE(String, 0xffff)
